@@ -26,7 +26,8 @@ COL_NAME = 1
 COL_ROLE = 2
 COL_CHILDCOUNT = 3
 COL_FILLED = 4
-COL_ACC = 5
+COL_DUMMY = 5
+COL_ACC = 6
 
 class AccessibleModel(gtk.TreeStore, Tools):
   '''
@@ -40,15 +41,31 @@ class AccessibleModel(gtk.TreeStore, Tools):
   resident in the model. This helps with faster searching.
   @type acc_cache: list
   '''
+  __gsignals__ = {'row-filled' : 
+                  (gobject.SIGNAL_RUN_FIRST,
+                   gobject.TYPE_NONE, 
+                   (gobject.TYPE_PYOBJECT,)),
+                  'start-populating' :
+                    (gobject.SIGNAL_RUN_FIRST,
+                     gobject.TYPE_NONE, 
+                     ()),
+                  'end-populating' :
+                    (gobject.SIGNAL_RUN_FIRST,
+                     gobject.TYPE_NONE, 
+                     ())}
+
   def __init__(self, desktop_acc):
     '''
     Initializes the L{AccessibleModel} and all of the needed instant variables.
     Connects required signals.
     '''
     self.acc_cache = [desktop_acc]
-    gtk.TreeStore.__init__(self, gtk.gdk.Pixbuf, str, str, str, bool, object)
+    gtk.TreeStore.__init__(self, gtk.gdk.Pixbuf, str, str, str, bool, bool, object)
     self.connect('row-changed', self._onRowChanged)
+    self.connect('row-filled', self._onRowFilled)
     self.desktop = desktop_acc
+    self._path_to_populate = None
+    self._populating_tasks = 0
 
   def _onRowChanged(self, model, path, iter):
     '''
@@ -158,33 +175,123 @@ class AccessibleModel(gtk.TreeStore, Tools):
     '''
     if (parent and parent.childCount > 0 and 
         not self.isMyApp(parent) and self.iter_n_children(iter) == 0):
-      row = self._buildRow(None, False)
+      row = self._buildRow(None, True)
       self.append(iter, row)
 
-  def popLevel(self, parent, iter):
+  def popLevel(self, iter):
     '''
     Populate a row with children rows, according to the row's accessible's children.
 
-    @param parent: The given row's accessible.
-    @type parent: L{Accessibility.Accessible}
     @param iter: Th iter of the row that needs to be populated
     @type iter: L{gtk.TreeIter}
-    
-    @return: A list of L{gtk.TreeIter}s that have been inserted.
-    @rtype: list
     '''
-    iters = []
-    if parent is None:
-      return iters
-    for child in parent:
-      if child is None:
-        row = self._buildRow(None, False, _('<dead>'))
-        citer = self.append(iter, row)
-      else:
-        row = self._buildRow(child, False)
-        citer = self.append(iter, row)
-      iters.append(citer)
-    return iters
+    if iter:
+      row_reference = gtk.TreeRowReference(self, self.get_path(iter))
+    else:
+      row_reference = None
+    self._populating_tasks += 1
+    if self._populating_tasks == 1:
+      self.emit('start-populating')
+    gobject.idle_add(self._popOnIdle, row_reference)
+
+  def _popOnIdle(self, row_reference):
+    '''
+    Idle callback for populating the children of a given row reference.
+    
+    @param row_reference: The parent row reference.
+    @type row_reference: gtk.TreeRowReference
+    
+    @return: False if task is done (to stop handler).
+    @rtype: boolean
+    '''
+    remove_iter = None
+    iter = None
+    parent = self.desktop
+
+    if row_reference:
+      if not row_reference.valid():
+        self._endPopTask()
+        return False
+      iter = self.get_iter(row_reference.get_path())
+      parent = self[iter][COL_ACC]
+      if self[(row_reference.get_path())+(0,)][COL_DUMMY]:
+        remove_iter = self.iter_children(iter)
+
+    already_populated_num = self.iter_n_children(iter)
+
+    if already_populated_num >= parent.childCount and \
+          not remove_iter:
+      if iter:
+        self[iter][COL_FILLED] = True
+      self.emit('row-filled', iter)
+      self._endPopTask()
+      return False
+    elif remove_iter:
+      already_populated_num -= 1
+    try:
+      child = parent.getChildAtIndex(already_populated_num)
+    except LookupError:
+      child = None
+
+    row = self._buildRow(child)
+    self.append(iter, row)
+
+    if remove_iter:
+      self.remove(remove_iter)
+    return True
+
+  def _endPopTask(self):
+    '''
+    Convinience function for stopping a populating task.
+    '''
+    self._populating_tasks -= 1
+    if self._populating_tasks == 0:
+      self.emit('end-populating')
+
+  def popToPath(self, path):
+    '''
+    Populate the model with accessible nodes up to given path.
+    
+    @param path: Path to populate model to.
+    @type path: tuple
+    '''
+    if not self._walkThroughFilled(path):
+      self._path_to_populate = path
+    else:
+      self.emit('row-filled', self.get_iter(path))
+      self._path_to_populate = None
+
+  def _walkThroughFilled(self, path):
+    '''
+    Reach node in path that is not populated yet, and populate it.
+    
+    @param path: Path to fill.
+    @type path: tuple
+    
+    @return: True if entire path is populated.
+    @rtype: boolean
+    '''
+    for i in xrange(1, len(path)):
+      if not self[path[:i]][COL_FILLED]:
+        self.popLevel(self.get_iter(path[:i]))
+        return False
+    return True      
+
+  def _onRowFilled(self, model, iter):
+    '''
+    Callback for "row-filled" signal. If there is a specific path we need to populate,
+    we continue populating the next node.
+    
+    @param model: Model that emitted signal (self).
+    @type model: L{AccessibleModel}
+    @param iter: Iter of row that has been populated.
+    @type iter: gtk.TreeIter
+    '''
+    if iter and self._path_to_populate:
+      path = self.get_path(iter)
+      if self._path_to_populate[:len(path)] == path:
+        if self._walkThroughFilled(self._path_to_populate):
+          self._path_to_populate = None
 
   def getAccPath(self, acc):
     '''
@@ -202,7 +309,10 @@ class AccessibleModel(gtk.TreeStore, Tools):
     child = acc
     while child.parent:
       try:
-        path = (child.getIndexInParent(),) + path
+        index_in_parent = child.getIndexInParent()
+        if index_in_parent < 0:
+          break
+        path = (index_in_parent,) + path
       except Exception, e:
         return None
       child = child.parent
@@ -212,7 +322,7 @@ class AccessibleModel(gtk.TreeStore, Tools):
       return None
     return path
 
-  def _buildRow(self, accessible, filled, name=None):
+  def _buildRow(self, accessible, dummy=False):
     '''
     Wrapper for building a row in the tree. Use this method instead of trying
     to construct the row by hand as it will be synced with the design of the
@@ -220,23 +330,23 @@ class AccessibleModel(gtk.TreeStore, Tools):
 
     @param accessible: Accessible object
     @type accessible: L{Accessibility.Accessible}
-    @param filled: Should the row be considered populated?
-    @type filled: boolean
-    @param name: Optional name to use as an override for the name gotten from
-      the accessible
-    @type name: string
+    @param dummy: Is this a dummy row?
+    @type dummy: boolean
     '''
     if accessible is not None:
       icon = getIcon(accessible)
-      name = name or accessible.name
+      name = accessible.name
       role = accessible.getLocalizedRoleName()
       count = str(accessible.childCount)
     else:
       icon = None
-      name = name or None
+      if not dummy:
+        name = _('<dead>')
+      else:
+        name = None
       role = None
       count = None
-    return [icon, name, role, count, filled, accessible]
+    return [icon, name, role, count, False, dummy, accessible]
 
 class AccessibleTreeView(gtk.TreeView, Tools):
   '''
@@ -254,15 +364,18 @@ class AccessibleTreeView(gtk.TreeView, Tools):
   @ivar model: The data model of this treeview.
   @type model: L{AccessibleModel}
   '''
-  def __init__(self):
+  def __init__(self, node):
     '''
     Initialize the treeview. Build the proper columns.
     Connect all of the proper signal handlers and at-spi event handlers.
+    
+    @param node: The main application node.
+    @type node: L{Node}
     '''
     gtk.TreeView.__init__(self)
 
     self.desktop = pyatspi.Registry.getDesktop(0)
-    self.node = Node()
+    self.node = node
     self.node.update(self.desktop)
     self._changed_handler = self.node.connect('accessible_changed',
                                              self._onAccChanged)
@@ -296,7 +409,12 @@ class AccessibleTreeView(gtk.TreeView, Tools):
     tvc.set_cell_data_func(crt, self._accCellDataFunc)
     self.append_column(tvc)
 
-    self.refreshTopLevel()
+    self.model.connect('row-filled', self._onRowFilled)
+    self.model.connect('start-populating', self._onStartPop)
+    self.model.connect('end-populating', self._onEndPop)
+    self._path_to_expand = None
+
+    self._refreshTopLevel()
 
     selection = self.get_selection()
     selection.unselect_all()
@@ -306,18 +424,33 @@ class AccessibleTreeView(gtk.TreeView, Tools):
     pyatspi.Registry.registerEventListener(self._accEventChildChanged, 
                                            'object:children-changed')
 
+    self.action_group = gtk.ActionGroup('TreeActions')
+    self.action_group.add_actions([
+        ('RefreshAll', gtk.STOCK_REFRESH, None,
+         None, 'Refresh all', self._refreshTopLevel),
+        ('RefreshCurrent', gtk.STOCK_JUMP_TO, _('Refresh _Current'),
+         None, 'Refresh selected node\'s children', self._refreshCurrentLevel)])  
 
-  def refreshTopLevel(self):
+    self.refresh_current_action = self.action_group.get_action('RefreshCurrent')
+    self.refresh_current_action.set_sensitive(False)
+
+  def _refreshTopLevel(self, action=None):
     '''
     Refreshes the entire tree at the desktop level.
+
+    @param action: Action object that emitted this signal, if any.
+    @type: gtk.Action
     '''
     self.model.clear()
-    iters = self.model.popLevel(self.desktop, None)
+    self.model.popLevel(None)
     # iter over all apps in the desktop too
     
-  def refreshCurrentLevel(self):
+  def _refreshCurrentLevel(self, action):
     '''
     Refreshes the current level. Selects and expands the parent of the level.
+
+    @param action: Action object that emitted this signal, if any.
+    @type: gtk.Action
     '''
     selection = self.get_selection()
     model, iter = selection.get_selected()
@@ -342,12 +475,9 @@ class AccessibleTreeView(gtk.TreeView, Tools):
     # don't repopulate if it has been filled before
     if self.model[iter][COL_FILLED]:
       return
-    self.model[iter][COL_FILLED] = True
     acc = self.model[iter][COL_ACC]
     # populate this level
-    self.model.popLevel(acc, iter)
-    # clear the dummy node from this level
-    self.model.remove(self.model.iter_children(iter))
+    self.model.popLevel(iter)
 
   def _accEventChildChanged(self, event):
     '''
@@ -360,10 +490,10 @@ class AccessibleTreeView(gtk.TreeView, Tools):
       # Bad karma
       return
     if self.model.isInModel(event.source):
-      path = self.model.getAccPath(event.source)
-      if path:
+      try:
+        path = self.model.getAccPath(event.source)
         iter = self.model.get_iter(path)
-      else:
+      except:
         iter = None
       if (event.source == self.desktop) or \
             (path and self.model[path][COL_FILLED]):
@@ -390,17 +520,15 @@ class AccessibleTreeView(gtk.TreeView, Tools):
       new_child = added.pop()
     except KeyError:
       return
+    row = self.model._buildRow(new_child)
     if new_child is None:
-      row = self.model._buildRow(new_child, False, name=_('<dead>'))
       self.model.append(iter, row)
-      return
     else:
-      row = self.model._buildRow(new_child, False)
       self.model.insert(iter, new_child.getIndexInParent(), row)
-    # We do this because an application won't have an icon loaded in 
-    # the window manager when it is first registered to at-spi
-    if new_child == new_child.getApplication():
-      gobject.timeout_add(1000, self._refreshIcon, new_child)
+      # We do this because an application won't have an icon loaded in 
+      # the window manager when it is first registered to at-spi
+      if new_child == new_child.getApplication():
+        gobject.timeout_add(1000, self._refreshIcon, new_child)
     
   def _refreshIcon(self, app):
     '''
@@ -446,7 +574,7 @@ class AccessibleTreeView(gtk.TreeView, Tools):
     @type iter: L{gtk.TreeIter}
     '''
     if not iter:
-      self.refreshTopLevel()
+      self._refreshTopLevel()
       return
     child_iter = self.model.iter_children(iter)
     while child_iter:
@@ -472,16 +600,22 @@ class AccessibleTreeView(gtk.TreeView, Tools):
     signal.
     @type selection: L{gtk.TreeSelection}
     '''
+    self._path_to_expand = None
     model, iter = selection.get_selected()
-    try:
+    if iter:
       new_acc = model[iter][COL_ACC]
-    except TypeError:
+      self.refresh_current_action.set_sensitive(True)
+    else:
       new_acc = self.desktop
+      self.refresh_current_action.set_sensitive(False)
     if new_acc == self.node.acc:
       return
     self.node.handler_block(self._changed_handler)
     self.node.update(new_acc)
     self.node.handler_unblock(self._changed_handler)
+    if iter:
+      path = model.get_path(iter)
+      self.node.tree_path = list(path[1:])
 
   def _onAccChanged(self, node, acc):
     '''
@@ -498,14 +632,77 @@ class AccessibleTreeView(gtk.TreeView, Tools):
     path = self.model.getAccPath(acc)
     if not path:
       return
+    if len(path) >= 1:
+      self.selectNodeAtPath(path)
+      self.node.tree_path = list(path[1:])
+
+  def selectNodeAtPath(self, path):
+    '''
+    Select the node at the current path. The path does not need to exist in the model,
+    only in the accessibles tree. The model will get populated accordingly.
+
+    @param path: The path to select.
+    @type path: tuple
+    '''
+    try:
+      dummy = self.model[path][COL_DUMMY]
+    except:
+      dummy = True
+    if dummy:
+      self._path_to_expand = path
+      self.model.popToPath(path)
+    else:
+      self._selectExistingPath(path)
+
+  def _onRowFilled(self, model, iter):
+    '''
+    Callback for "row-filled" (populated) signal. Used for selecting a child node in 
+    the given iter if L{selectNodeAtPath} was called on one of the given 
+    iter's children.
+    
+    @param model: Model that emitted this signal.
+    @type model: L{AccessibleModel}
+    @param iter: Iter of row that was populated with children.
+    @type iter: gtk.TreeIter
+    '''
+    if iter and self._path_to_expand and \
+          self._path_to_expand[:-1] == model.get_path(iter):
+      self._selectExistingPath(self._path_to_expand)
+      self._path_to_expand = None
+
+  def _selectExistingPath(self, path):
+    '''
+    Select a path that already exists. Expand, scroll, and select.
+    
+    @param path: Path to select.
+    @type path: tuple
+    '''
     if len(path) > 1:
       self.expand_to_path(path[:-1])
     self.scroll_to_cell(path)
     selection = self.get_selection()
-    try:
-      selection.select_iter(self.model.get_iter(path))
-    except ValueError:
-      pass
+    selection.select_path(path)
+    
+
+  def _onStartPop(self, model):
+    '''
+    Callback for when the model is populating, changes the cursor to a watch.
+    
+    @param model: Model that emitted the signal.
+    @type model: L{AccessibleModel}
+    '''
+    if self.window:
+      self.window.set_cursor(gtk.gdk.Cursor(gtk.gdk.WATCH))
+
+  def _onEndPop(self, model):
+    '''
+    Callback for when the model stops populating, changes the cursor to an arrow.
+    
+    @param model: Model that emitted the signal.
+    @type model: L{AccessibleModel}
+    '''
+    if self.window:
+      self.window.set_cursor(gtk.gdk.Cursor(gtk.gdk.TOP_LEFT_ARROW))
 
   def _accCellDataFunc(self, tvc, cellrenderer, model, iter):
     '''
